/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.io;

import java.io.PrintWriter;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.opengion.fukurou.system.HybsConst;				// 6.1.0.0 (2014/12/26)
import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.HybsEntry;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.db.DBTableModel;
import org.opengion.hayabusa.db.DBColumn;

/**
 * 区切り文字指定(初期値:ﾀﾌﾞ)ｾﾞﾛｶﾝﾏﾌｧｲﾙの書き出しｸﾗｽです。
 *
 * ﾗﾍﾞﾙ、名前、ﾃﾞｰﾀの出力部のみｵｰﾊﾞｰﾗｲﾄﾞすれば、各種出力ﾌｫｰﾏｯﾄに合わせた
 * ｻﾌﾞｸﾗｽを実現する事が可能です。
 * ｾﾞﾛｶﾝﾏﾌｧｲﾙとは、EXCELのｾﾞﾛｻﾌﾟﾚｽ対策として、頭ｾﾞﾛの文字型ﾃﾞｰﾀを出力する
 * 時に、先頭にｶﾝﾏ(')を付けて、ｾﾞﾛが削除(見えなくなる)現象を抑止しているﾌｧｲﾙです。
 *
 * このｸﾗｽは、可変長ﾀﾌﾞ区切り文字ﾌｧｲﾙの出力機能を実現しています。
 *
 * @og.group ﾌｧｲﾙ出力
 *
 * @version	4.0
 * @author	Kazuhiko Hasegawa
 * @since	JDK5.0,
 */
public abstract class AbstractTableWriter implements TableWriter {
	/** このﾌﾟﾛｸﾞﾗﾑのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "8.5.2.0 (2023/07/14)" ;

	/** ｼｽﾃﾑの改行ｺｰﾄﾞを設定します。*/
	protected static final String CR		 = HybsConst.CR;					// 6.1.0.0 (2014/12/26) refactoring
	/** StringBilderなどの初期値を設定します。	{@value} */
	protected static final int BUFFER_MIDDLE = HybsConst.BUFFER_MIDDLE;			// 6.1.0.0 (2014/12/26) refactoring

	/** dbType の簡易的な設定	{@value} */
	public static final int STRING  = 0;
	/** dbType の簡易的な設定	{@value} */
//	public static final int NVAR    = 1;										// 8.5.2.0 (2023/07/14) Delete
	/** dbType の簡易的な設定	{@value} */
	public static final int NUMBER  = 2;

	/** 出力対象のｶﾗﾑ数 */
	protected int	numberOfColumns		= -1;									// 4.0.0 (2005/01/31) 出力対象のｶﾗﾑ数
	/** ﾃｰﾌﾞﾙﾓﾃﾞﾙ */
	protected DBTableModel	table		;
	/** DBｶﾗﾑ */
	protected DBColumn[]	dbColumn	;										// table に対するｶﾗﾑｵﾌﾞｼﾞｪｸﾄ(ｷｬｯｼｭ)
	/** ｶﾗﾑ数 */
	protected int[]			clmNo		;										// 出力対象のｶﾗﾑ番号配列
	/** DBﾀｲﾌﾟ */
	protected int[]			dbType		;										// 5.1.6.0 (2010/05/01) dbType の簡易的な設定
	/** KeyLabel出力 */
	protected boolean[]		isKeyLbl	;										// 6.2.3.0 (2015/05/01) KeyLabel出力

	private String	separator		= TAB_SEPARATOR;							// 項目区切り文字
	private String	headerSequence	= HybsSystem.sys( "WRITER_HEADER_SEQUENCE" ) ;
	private boolean append			;											// 3.5.4.2 (2003/12/15)
	private String	lang			;
	private String	columns			;											// 4.0.0 (2005/11/30) 外部指定のｶﾗﾑ名
	// 6.1.0.0 (2014/12/26) 書き込み対象外のｶﾗﾑ列を、外部(タグ)より指定する。
	private String	omitNames		;											// 6.1.0.0 (2014/12/26)
	private String	encode			;											// 3.5.4.5 (2004/01/23) ｴﾝｺｰﾄﾞ文字列指定

	private boolean	useNumber		= true;										// 3.7.0.2 (2005/02/14) 行番号情報を、出力する(true)/しない(false)を指定
	private boolean	useRenderer		;											// 5.2.1.0 (2010/10/01)

	private boolean	useDebug		;											// 6.1.0.0 (2014/12/26) ﾃﾞﾊﾞｯｸﾞ情報の出力するかどうか

	/**
	 * ﾃﾞﾌｫﾙﾄｺﾝｽﾄﾗｸﾀｰ
	 *
	 * @og.rev 6.4.2.0 (2016/01/29) PMD refactoring. Each class should declare at least one constructor.
	 */
	protected AbstractTableWriter() { super(); }		// これも、自動的に呼ばれるが、空のﾒｿｯﾄﾞを作成すると警告されるので、明示的にしておきます。

	/**
	 * DBTableModel から 各形式のﾃﾞｰﾀを作成して、PrintWriter に書き出します。
	 * このﾒｿｯﾄﾞは、EXCEL 書き出し時に使用します。
	 *
	 * @see	#isExcel()
	 */
	@Override	// TableWriter
	public abstract void writeDBTable();

	/**
	 * DBTableModel から ﾃﾞｰﾀを作成して、PrintWriter に書き出します。
	 *
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	@Override	// TableWriter
	public abstract void writeDBTable( final PrintWriter writer );

	/**
	 * numberOfColumns と DBColumn を初期化します。
	 * 内部的に、DBTableModel、lang , columns を使用して、
	 * numberOfColumns 、dbColumn、clmNo、dbType の値を初期化します。
	 * ｶﾗﾑが1項目もない場合、言語(lnag)が未指定、DBTableModelが未指定(null)
	 * の場合は、false を返します。その場合は、以下の処理は正常に行えません。
	 * ﾃﾞｰﾀが0件の場合は、処理を行います。通常、ﾍｯﾀﾞｰのみのﾌｧｲﾙを
	 * 作成することになります。(これを雛形として、取込ﾃﾞｰﾀを登録する事が可能)
	 *
	 * 6.0.1.2 (2014/08/08)
	 *  clmNo 配列に、-1 がｾｯﾄされた場合、DBTableModel に存在しないｶﾗﾑとして
	 *  そのｶﾗﾑ列は、書き込みしません。,,,, などを指定する事で、ｶﾗﾑを
	 *  飛ばして書き込むことが可能です。
	 *  refFileURL、refFilename、refSheetName とともに使用すれば、ある程度の
	 *  ﾚｲｱｳﾄ設定が可能です。
	 *
	 * @og.rev 4.0.0.0 (2005/12/31) 外部指定のｶﾗﾑ名を使用
	 * @og.rev 5.1.6.0 (2010/05/01) DbType の初期値を設定
	 * @og.rev 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばしできる機能を追加
	 * @og.rev 6.0.2.0 (2014/09/19) EXCEL新機能対応(２回呼ばれない様にする)
	 * @og.rev 6.1.0.0 (2014/12/26) omitNames 属性を追加
	 * @og.rev 6.2.3.0 (2015/05/01) writeKeyLabel 属性を追加
	 * @og.rev 8.5.2.0 (2023/07/14) 一部の機能廃止による修正(問合・ﾄﾗﾌﾞﾙ 0200010980)
	 *
	 * @return	初期化成功：true / 失敗：false
	 */
	protected boolean createDBColumn() {
		if( numberOfColumns >= 0 ) { return true; }		// 6.0.2.0 (2014/09/19) EXCEL新機能対応(2回呼ばれない様にする)

		boolean rtnFlag = false;
		if( lang != null && table != null ) {
			// 6.4.1.1 (2016/01/16) PMD refactoring. Avoid if (x != y) ..; else ..;
			if( columns == null ) {
				numberOfColumns = table.getColumnCount();
				clmNo = new int[numberOfColumns];
				for( int i=0; i<numberOfColumns; i++ ) {
					clmNo[i] = i;
				}
			}
			else {
				final String[] clms = StringUtil.csv2Array( columns );
				numberOfColumns = clms.length;
				clmNo = new int[numberOfColumns];
				for( int i=0; i<numberOfColumns; i++ ) {
					final int no = table.getColumnNo( clms[i],false );	// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし
					clmNo[i] = no;
				}
			}

			// 6.1.0.0 (2014/12/26) 書き込み対象外のｶﾗﾑ列を、外部(ﾀｸﾞ)より指定する。(omitNames属性)
			if( omitNames != null ) {
				final String[] omit = StringUtil.csv2Array( omitNames );
				for( int j=0; j<omit.length; j++ ) {
					// omitNames は、ｶﾗﾑ飛ばしの機能を利用
					final int no = table.getColumnNo( omit[j],false );
					if( no >= 0 ) {
						for( int i=0; i<numberOfColumns; i++ ) {
							if( clmNo[i] == no ) { clmNo[i] = -1 ; break; }		// omitｶﾗﾑは、-1 を設定する。
						}
					}
				}
			}

			dbColumn = table.getDBColumns();
			rtnFlag  = numberOfColumns > 0 ;				// ｶﾗﾑが1項目以上あれば成功

			// 5.1.6.0 (2010/05/01) DbType の初期値を設定する。
			dbType   = new int[numberOfColumns];
			isKeyLbl = new boolean[numberOfColumns];		// 6.2.3.0 (2015/05/01) KeyLabel出力
			for( int i=0; i<numberOfColumns; i++ ) {
				final int clm = clmNo[i];					// 6.2.3.0 (2015/05/01)
				// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし の場合は、clmNo[i] には、-1 が設定されている。
				if( clm < 0 ) { continue; }

				final String type = dbColumn[clm].getDbType();
//				if( "NVAR".equals( type ) ) {									// 8.5.2.0 (2023/07/14) Delete
//					dbType[i] = NVAR;											// 8.5.2.0 (2023/07/14) Delete
//				}																// 8.5.2.0 (2023/07/14) Delete
//				else if( "S9".equals( type ) || "R".equals( type )  ) {			// 8.5.2.0 (2023/07/14) Modify
				if( "S9".equals( type ) || "R".equals( type )  ) {
					dbType[i] = NUMBER;
				}
				else {
					dbType[i] = STRING;
				}

				// 6.2.3.0 (2015/05/01) KeyLabel出力
				isKeyLbl[i] = dbColumn[clm].isWriteKeyLabel();					// 6.2.3.0 (2015/05/01) KeyLabel出力
			}
		}
		return rtnFlag;
	}

	/**
	 * PrintWriter に DBTableModelのﾍｯﾀﾞｰ情報を書き込みます。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 *
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected void writeHeader( final PrintWriter writer ) {
		// ここでは処理を行いません。
	}

	/**
	 * PrintWriter に DBTableModelのﾍｯﾀﾞｰ情報を書き込みます。
	 * ここでは、ﾗﾍﾞﾙ情報と同じﾃﾞｰﾀを出力しますが、# を外します。
	 * また、ﾗﾍﾞﾙを文字列としてﾀﾞﾌﾞﾙｺｰﾃｰｼｮﾝでくくります。
	 *
	 * @og.rev 7.2.6.1 (2020/07/17) H:Header(ﾍｯﾀﾞｰ)属性追加。
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected void writeLabel2( final DBTableModel table,final PrintWriter writer ) {
		if( useNumber ) {
			writer.print( separator );
		}

		for( int i=0; i<numberOfColumns; i++ ) {
			if( i != 0 ) { writer.print( separator ); }
			final int clm = clmNo[i];
			if( clm < 0 ) { continue; }						// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし
			final String val = StringUtil.tagCut( dbColumn[clm].getLabel() );	// 6.2.4.2 (2015/05/29)
			writer.print( quotation( val ) );
			// keyLabelの場合は、後ろのｶﾗﾑのﾗﾍﾞﾙを付けません。
			if( isKeyLbl[i] ) { writer.print( separator ); }	// 6.2.3.0 (2015/05/01) writeKeyLabel 属性
		}
		writer.println();
	}

	/**
	 * PrintWriter に DBTableModelのﾗﾍﾞﾙ情報を書き込みます。
	 * 第一ｶﾗﾑ目は、ﾗﾍﾞﾙ情報を示す "#Label" を書き込みます。
	 * この行は、出力形式に無関係に、TableWriter.TAB_SEPARATOR で区切られます。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 * @og.rev 3.7.0.2 (2005/02/14) 行番号情報を、出力する(true)/しない(false)を指定
	 * @og.rev 4.0.0.0 (2005/12/31) 外部指定のｶﾗﾑ名を使用
	 * @og.rev 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばしできる機能を追加
	 * @og.rev 6.2.3.0 (2015/05/01) writeKeyLabel 属性を追加
	 * @og.rev 6.2.4.2 (2015/05/29) StringUtil#tagCut(String) をﾗﾍﾞﾙに適用します。
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected void writeLabel( final DBTableModel table,final PrintWriter writer ) {
		if( useNumber ) {
			writer.print( "#Label" );
			writer.print( separator );
		}
		else {
			writer.print( "#" );
		}

		for( int i=0; i<numberOfColumns; i++ ) {
			if( i != 0 ) { writer.print( separator ); }
			final int clm = clmNo[i];
			if( clm < 0 ) { continue; }						// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし
			final String val = StringUtil.tagCut( dbColumn[clm].getLabel() );	// 6.2.4.2 (2015/05/29)
			writer.print( val );
			// keyLabelの場合は、後ろのｶﾗﾑのﾗﾍﾞﾙを付けません。
			if( isKeyLbl[i] ) { writer.print( separator ); }	// 6.2.3.0 (2015/05/01) writeKeyLabel 属性
		}
		writer.println();
	}

	/**
	 * PrintWriter に DBTableModelの項目名情報を書き込みます。
	 * 第一ｶﾗﾑ目は、項目名情報を示す "#Name" を書き込みます。
	 * useNumber=fale の場合は、最初のｶﾗﾑ名の先頭に、# が書き込まれます。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 * @og.rev 3.7.0.2 (2005/02/14) 行番号情報を、出力する(true)/しない(false)を指定
	 * @og.rev 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばしできる機能を追加
	 * @og.rev 6.2.3.0 (2015/05/01) writeKeyLabel 属性を追加
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected void writeName( final DBTableModel table,final PrintWriter writer ) {
		if( useNumber ) {
			writer.print( "#Name" );
			writer.print( separator );
		}
		else {
			writer.print( "#" );
		}

		for( int i=0; i<numberOfColumns; i++ ) {
			if( i != 0 ) { writer.print( separator ); }
			final int clm = clmNo[i];
			if( clm < 0 ) { continue; }						// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし
			writer.print( table.getColumnName(clm) );
			// keyLabelの場合は、後ろのｶﾗﾑの名前を付けません。
			if( isKeyLbl[i] ) { writer.print( separator ); }	// 6.2.3.0 (2015/05/01) writeKeyLabel 属性
		}
		writer.println();
	}

	/**
	 * PrintWriter に DBTableModelのｻｲｽﾞ情報を書き込みます。
	 * 第一ｶﾗﾑ目は、ｻｲｽﾞ情報を示す "#Size" を書き込みます。
	 * この行は、出力形式に無関係に、TableWriter.TAB_SEPARATOR で区切られます。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 * @og.rev 3.5.5.5 (2004/04/23) DBColumn の size と maxlength の 意味を変更
	 * @og.rev 3.7.0.2 (2005/02/14) 行番号情報を、出力する(true)/しない(false)を指定
	 * @og.rev 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばしできる機能を追加
	 * @og.rev 6.2.3.0 (2015/05/01) writeKeyLabel 属性を追加
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected void writeSize( final DBTableModel table,final PrintWriter writer ) {
		if( useNumber ) {
			writer.print( "#Size" );
			writer.print( separator );
		}
		else {
			writer.print( "#" );
		}

		for( int i=0; i<numberOfColumns; i++ ) {
			if( i != 0 ) { writer.print( separator ); }
			final int clm = clmNo[i];
			if( clm < 0 ) { continue; }						// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし
			writer.print( dbColumn[clm].getTotalSize() );	// 4.0.0 (2005/01/31) ﾒｿｯﾄﾞ名変更
			// keyLabelの場合は、後ろのｶﾗﾑのｻｲｽﾞを付けません。
			if( isKeyLbl[i] ) { writer.print( separator ); }	// 6.2.3.0 (2015/05/01) writeKeyLabel 属性
		}
		writer.println();
	}

	/**
	 * PrintWriter に DBTableModelのｸﾗｽ名情報を書き込みます。
	 * 第一ｶﾗﾑ目は、ｻｲｽﾞ情報を示す "#Class" を書き込みます。
	 * この行は、出力形式に無関係に、TableWriter.TAB_SEPARATOR で区切られます。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 * @og.rev 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばしできる機能を追加
	 * @og.rev 6.2.3.0 (2015/05/01) writeKeyLabel 属性を追加
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected void writeClass( final DBTableModel table,final PrintWriter writer ) {
		if( useNumber ) {
			writer.print( "#Class" );
			writer.print( separator );
		}
		else {
			writer.print( "#" );
		}

		for( int i=0; i<numberOfColumns; i++ ) {
			if( i != 0 ) { writer.print( separator ); }
			final int clm = clmNo[i];
			if( clm < 0 ) { continue; }						// 6.0.1.2 (2014/08/08) ｶﾗﾑ飛ばし
			writer.print( dbColumn[clm].getClassName() );
			// keyLabelの場合は、後ろのｶﾗﾑのｸﾗｽを付けません。
			if( isKeyLbl[i] ) { writer.print( separator ); }	// 6.2.3.0 (2015/05/01) writeKeyLabel 属性
		}
		writer.println();
	}

	/**
	 * PrintWriter に ｾﾊﾟﾚｰﾀｰを書き込みます。
	 * 第一ｶﾗﾑ目は、ｻｲｽﾞ情報を示す "#----" を書き込みます。
	 * この行は、出力形式に無関係に、TableWriter.TAB_SEPARATOR で区切られます。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 * @og.rev 3.7.0.2 (2005/02/14) 行番号情報を、出力する(true)/しない(false)を指定
	 * @og.rev 6.2.3.0 (2015/05/01) writeKeyLabel 属性を追加
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected void writeSeparator( final DBTableModel table,final PrintWriter writer ) {
		final String sep = "----" ;
		if( useNumber ) {
			writer.print( "#----" );
			writer.print( separator );
		}
		else {
			writer.print( "#" );
		}

		for( int i=0; i<numberOfColumns; i++ ) {
			if( i != 0 ) { writer.print( separator ); }
			writer.print( sep );
			// keyLabelの場合は、後ろのｶﾗﾑのｾﾊﾟﾚｰﾀｰを付けません。
			if( isKeyLbl[i] ) { writer.print( separator ); }	// 6.2.3.0 (2015/05/01) writeKeyLabel 属性
		}
		writer.println();
	}

	/**
	 * PrintWriter に DBTableModelのﾃｰﾌﾞﾙ情報を書き込みます。
	 * このｸﾗｽでは、ﾃﾞｰﾀを ﾀﾞﾌﾞﾙｺｰﾃｰｼｮﾝ(")で囲みます。
	 * PrintWriter に DBTableModelのﾃｰﾌﾞﾙ情報を書き込みます。
	 *
	 * @og.rev 2.0.0.5 (2002/09/30) 先頭が0 でかつ数字ﾀｲﾌﾟ(S9 or R)でない場合に ' を出力するように修正。
	 * @og.rev 2.3.1.2 (2003/01/28) ﾃﾞｰﾀ出力時に、改行が余分に出される箇所を修正。
	 * @og.rev 3.1.0.0 (2003/03/20) DBColumn から、getDbType() ｷｰを直接取り出す
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 * @og.rev 3.3.3.1 (2003/07/18) ﾌｧｲﾙﾘｰﾄﾞ/ﾗｲﾄ時に後ろｽﾍﾟｰｽの除去を行います。
	 * @og.rev 3.7.0.2 (2005/02/14) 行番号情報を、出力する(true)/しない(false)を指定
	 * @og.rev 3.8.0.1 (2005/06/17) DBTypeが NVAR の場合は、元のUnicodeに戻します。
	 * @og.rev 5.1.6.0 (2010/05/01) DbType の初期値(dbType)を利用する。
	 * @og.rev 5.2.1.0 (2010/10/01) このﾒｿｯﾄﾞは、abstract 化します。
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	writer	PrintWriterｵﾌﾞｼﾞｪｸﾄ
	 */
	protected abstract void writeData( final DBTableModel table,final PrintWriter writer ) ;

	/**
	 * DBTableModel をｾｯﾄします。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 * @og.rev 3.5.4.2 (2003/12/15) lang 引数も同時に設定します。
	 *
	 * @param	table	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 * @param	lang	言語ｺｰﾄﾞ
	 */
	@Override	// TableWriter
	public void setDBTableModel( final DBTableModel table, final String lang ) {
		this.table = table;
		this.lang = lang;
	}

	/**
	 * 内部の DBTableModel を返します。
	 *
	 * @return	DBTableModelｵﾌﾞｼﾞｪｸﾄ
	 */
	@Override	// TableWriter
	public DBTableModel getDBTableModel() {
		return table;
	}

	/**
	 * DBTableModelの出力順をｾｯﾄします。
	 * Name,Label,Size,Class,Data の各ﾌｨｰﾙﾄﾞの頭文字のｱﾙﾌｧﾍﾞｯﾄで
	 * 出力順を設定します。
	 *
	 * ※ 7.2.6.1 (2020/07/17) H:Header(ﾍｯﾀﾞｰ)属性追加
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 *
	 * @param	hs	出力順 (LNSCD など)
	 */
	@Override	// TableWriter
	public void setHeaderSequence( final String hs ) {
		if( hs != null ) { headerSequence = hs ; }
	}

	/**
	 * DBTableModelの出力順を返します。
	 * Label,Name,Size,Class,Data の各ﾌｨｰﾙﾄﾞの頭文字のｱﾙﾌｧﾍﾞｯﾄで
	 * 出力順を設定します。
	 *
	 * なお、出力順に指定しない項目は出力されません
	 *
	 * @return	出力順(LNSCD など)
	 */
	@Override	// TableWriter
	public String getHeaderSequence() {
		return headerSequence ;
	}

	/**
	 * ﾃﾞｰﾀを書き込む場合の区切り文字をｾｯﾄします。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) 同期ﾒｿｯﾄﾞ(synchronized付き)を非同期に変更する。
	 *
	 * @param	sep	区切り文字
	 */
	@Override	// TableWriter
	public void setSeparator( final String sep ) {
		if( sep != null ) { this.separator = sep; }
	}

	/**
	 * ﾃﾞｰﾀを書き込む場合の区切り文字を返します。
	 *
	 * @return	区切り文字
	 */
	public String getSeparator() {
		return separator;
	}

	/**
	 * DBTableModelのﾃﾞｰﾀとして書き込むときに、追加ﾓｰﾄﾞで書き込むかどうかを設定します。
	 * 初期値は、false(新規ﾓｰﾄﾞ)です。
	 *
	 * @og.rev 3.5.4.2 (2003/12/15) 新規追加
	 *
	 * @param	flag	[true:追加ﾓｰﾄﾞ/false:新規ﾓｰﾄﾞ]
	 */
	@Override	// TableWriter
	public void setAppend( final boolean flag ) {
		append = flag;
	}

	/**
	 * DBTableModelのﾃﾞｰﾀとして書き込むときに、追加ﾓｰﾄﾞで書き込むかどうかを取得します。
	 * 初期値は、false(新規ﾓｰﾄﾞ)です。
	 *
	 * @og.rev 3.5.4.2 (2003/12/15) 新規追加
	 *
	 * @return	true(追加ﾓｰﾄﾞ)/false(新規ﾓｰﾄﾞ)
	 */
	@Override	// TableWriter
	public boolean isAppend() {
		return append ;
	}

	/**
	 * DBTableModelのﾃﾞｰﾀとして読み込むときのｼｰﾄ名を設定します。
	 * 初期値は、"Sheet1" です。
	 * これは、EXCEL追加機能として実装されています。
	 * ※ このｸﾗｽでは実装されていません。
	 *
	 * @og.rev 3.5.4.2 (2003/12/15) 新規追加
	 * @og.rev 3.5.4.3 (2004/01/05) 実装廃止(TableWriter_Excel へ移動)
	 * @og.rev 6.0.2.0 (2014/09/19) isExcel() で判定しているので、Exception は投げない。
	 *
	 * @param	sheetName	ｼｰﾄ名
	 */
	@Override	// TableWriter
	public void setSheetName( final String sheetName ) {
		final String errMsg = "sheetName は、EXCEL専用機能なので使用できません。[" + sheetName + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL雛型参考ﾌｧｲﾙのｼｰﾄ名を設定します。
	 * これは、EXCEL追加機能として実装されています。
	 *
	 * EXCELﾌｧｲﾙを書き出す時に、雛型として参照するｼｰﾄ名を指定します。
	 * これにより、複数の形式の異なるﾃﾞｰﾀを順次書き出したり(appendﾓｰﾄﾞを併用)する
	 * ことや、ｼｰﾄを指定して新規にEXCELを作成する場合にﾌｫｰﾑを設定する事が可能になります。
	 * 初期値は、null(第一ｼｰﾄ) です。
	 * ※ このｸﾗｽでは実装されていません。
	 *
	 * @og.rev 3.5.4.3 (2004/01/05) 新規追加
	 * @og.rev 6.0.2.0 (2014/09/19) isExcel() で判定しているので、Exception は投げない。
	 *
	 * @param	sheetName	ｼｰﾄ名
	 */
	@Override	// TableWriter
	public void setRefSheetName( final String sheetName )  {
		final String errMsg = "refSheetName は、EXCEL専用機能なので使用できません。[" + sheetName + "]";
		System.err.println( errMsg );
	}

	/**
	 * このｸﾗｽが、EXCEL対応機能を持っているかどうかを返します。
	 *
	 * EXCEL対応機能とは、ｼｰﾄ名のｾｯﾄ、雛型参照ﾌｧｲﾙ名のｾｯﾄ、
	 * 書き込み元ﾌｧｲﾙのFileｵﾌﾞｼﾞｪｸﾄ取得などの、特殊機能です。
	 * 本来は、ｲﾝﾀｰﾌｪｰｽを分けるべきと考えますが、taglib ｸﾗｽ等の
	 * 関係があり、問い合わせによる条件分岐で対応します。
	 *
	 * @og.rev 3.5.4.3 (2004/01/05) 新規追加
	 *
	 * @return	EXCEL対応機能を持っているかどうか(ここでは、false固定です)
	 */
	@Override	// TableWriter
	public boolean isExcel() {
		return false;
	}

	/**
	 * 出力先ﾃﾞｨﾚｸﾄﾘとﾌｧｲﾙ名をｾｯﾄします。
	 * これは、EXCEL追加機能として実装されています。
	 * ※ このｸﾗｽでは実装されていません。
	 * このﾒｿｯﾄﾞでは、必ず、UnsupportedOperationException が、throw されます。
	 *
	 * @og.rev 3.5.4.3 (2004/01/05) 新規作成
	 * @og.rev 6.0.2.0 (2014/09/19) ﾃﾞｨﾚｸﾄﾘとﾌｧｲﾙを分けて管理します。
	 *
	 * @param	directory	出力先ﾃﾞｨﾚｸﾄﾘ名
	 * @param	filename	出力先ﾌｧｲﾙ名
	 */
	@Override	// TableWriter
	public void setFilename( final String directory , final String filename ) {
		final String errMsg = "filename は、メソッドは、EXCEL追加機能ですので、使用できません。";
		throw new UnsupportedOperationException( errMsg );
	}

	/**
	 * EXCEL雛型参考ﾌｧｲﾙ名をｾｯﾄします。(DIR + Filename)
	 * これは、EXCEL追加機能として実装されています。
	 * ※ このｸﾗｽでは実装されていません。
	 * このﾒｿｯﾄﾞでは、必ず、UnsupportedOperationException が、throw されます。
	 *
	 * @og.rev 3.5.4.3 (2004/01/05) 新規作成
	 * @og.rev 6.0.2.0 (2014/09/19) isExcel() で判定しているので、Exception は投げない。
	 *
	 * @param	filename	EXCEL雛型参考ﾌｧｲﾙ名
	 */
	@Override	// TableWriter
	public void setRefFilename( final String filename ) {
		final String errMsg = "refFilename は、EXCEL専用機能なので使用できません。[" + filename + "]";
		System.err.println( errMsg );
	}

	/**
	 * 雛形のｼｰﾄ名を、そのまま使用する(true)か、新規、または、外部指定のｼｰﾄ名を使用する(false)を指定します。(初期値:false[外部指定のｼｰﾄ名を使用])。
	 *
	 * ※ Ver5では、追記ﾓｰﾄﾞ時に、指定ｼｰﾄが存在した場合は上書きします(初期値:false[上書きしない])。5.9.12.1 (2016/09/09)
	 *    Ver6では、追記ﾓｰﾄﾞ時に、雛形を指定できないため、雛形のｼｰﾄ名を、そのまま使用する(true)か、
	 *    新規、または、外部指定のｼｰﾄ名を使用する(false)を指定する属性になります。
	 *
	 * @og.rev 6.5.0.0 (2016/09/30) sheetOverwrite で、雛形ｼｰﾄの使用時に、元のｼｰﾄ名を使用します。
	 *
	 * @param	flag	元のｼｰﾄ名を使用するかどうか[true:使用する/false:新規、または、外部指定のｼｰﾄ名を使用]
	 */
	@Override	// TableWriter
	public void setSheetOverwrite( final boolean flag ) {
		final String errMsg = "sheetOverwrite は、このメソッドは、EXCEL追加機能ですので、使用できません。";
		System.err.println( errMsg );
	}

	/**
	 * EXCELで、出力処理の最後にｾﾙの計算式の再計算をさせるｼｰﾄ名をCSV形式で指定します。
	 *
	 * @og.rev 6.5.0.0 (2016/09/30) recalcSheetName で、ｾﾙ内の計算式を再計算させるｼｰﾄ名を指定。5.9.12.1 (2016/09/09)
	 *
	 * @param	sheet	対象ｼｰﾄ名をCSV形式で指定
	 */
	@Override	// TableWriter
	public void setRecalcSheetName( final String sheet ){
		final String errMsg = "recalcSheetName は、メソッドは、EXCEL追加機能ですので、使用できません。";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時のﾃﾞﾌｫﾙﾄﾌｫﾝﾄ名を設定します。
	 * これは、EXCEL追加機能として実装されています。
	 *
	 * EXCELﾌｧｲﾙを書き出す時に、ﾃﾞﾌｫﾙﾄﾌｫﾝﾄ名を指定します。
	 * ﾌｫﾝﾄ名は、EXCELのﾌｫﾝﾄ名をそのまま使用してください。
	 * 内部的に、POI の org.apache.poi.hssf.usermodel.HSSFFont#setFontName( String )
	 * に設定されます。
	 * 初期値は、ｼｽﾃﾑﾘｿｰｽ の TABLE_WRITER_DEFAULT_FONT_NAME です。
	 *
	 * ※ このｸﾗｽでは実装されていません。
	 * このﾒｿｯﾄﾞでは、必ず、UnsupportedOperationException が、throw されます。
	 *
	 * @og.rev 3.8.5.3 (2006/08/07) 新規追加
	 * @og.rev 6.0.2.0 (2014/09/19) isExcel() で判定しているので、Exception は投げない。
	 *
	 * @param	fontName	ﾌｫﾝﾄ名
	 */
	@Override	// TableWriter
	public void setFontName( final String fontName ) {
		final String errMsg = "fontName は、EXCEL専用機能なので使用できません。[" + fontName + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時のﾃﾞﾌｫﾙﾄﾌｫﾝﾄﾎﾟｲﾝﾄ数を設定します。
	 * これは、EXCEL追加機能として実装されています。
	 *
	 * EXCELﾌｧｲﾙを書き出す時に、ﾃﾞﾌｫﾙﾄﾎﾟｲﾝﾄ数を指定します。
	 * 内部的に、POI の org.apache.poi.hssf.usermodel.HSSFFont#setFontHeightInPoints( short )
	 * に設定されます。
	 * 初期値は、ｼｽﾃﾑﾘｿｰｽ の TABLE_WRITER_DEFAULT_FONT_POINTS です。
	 *
	 * ※ このｸﾗｽでは実装されていません。
	 * このﾒｿｯﾄﾞでは、必ず、UnsupportedOperationException が、throw されます。
	 *
	 * @og.rev 3.8.5.3 (2006/08/07) 新規追加
	 * @og.rev 6.0.2.0 (2014/09/19) isExcel() で判定しているので、Exception は投げない。
	 *
	 * @param	point	ﾌｫﾝﾄﾎﾟｲﾝﾄ数
	 */
	@Override	// TableWriter
	public void setFontPoint( final short point ) {
		final String errMsg = "fontPoint は、EXCEL専用機能なので使用できません。[" + point + "]";
		System.err.println( errMsg );
	}

	/**
	 * 読み取り元ﾌｧｲﾙのｴﾝｺｰﾄﾞ文字列を指定します。
	 * ﾌｧｲﾙは、BufferedReader で受け取る為、本来は、ｴﾝｺｰﾄﾞは不要ですが、
	 * 固定長ﾌｧｲﾙの読み取り時のﾊﾞｲﾄｺｰﾄﾞ分割時に、指定のｴﾝｺｰﾄﾞで
	 * 分割する必要があります。(例えば、半角文字は、Shift_JIS では、1ﾊﾞｲﾄ)
	 *
	 * @og.rev 3.5.4.5 (2004/01/23) 新規作成
	 *
	 * @param	enc	ﾌｧｲﾙのｴﾝｺｰﾄﾞ文字列
	 */
	@Override	// TableWriter
	public void setEncode( final String enc ) {
		encode = enc;
	}

	/**
	 * 読み取り元ﾌｧｲﾙのｴﾝｺｰﾄﾞ文字列を取得します。
	 * ﾌｧｲﾙは、BufferedReader で受け取る為、本来は、ｴﾝｺｰﾄﾞは不要ですが、
	 * 固定長ﾌｧｲﾙの読み取り時のﾊﾞｲﾄｺｰﾄﾞ分割時に、指定のｴﾝｺｰﾄﾞで
	 * 分割する必要があります。(例えば、半角文字は、Shift_JIS では、1ﾊﾞｲﾄ)
	 *
	 * @og.rev 3.5.4.5 (2004/01/23) 新規作成
	 *
	 * @return	ﾌｧｲﾙのｴﾝｺｰﾄﾞ文字列
	 */
	protected String getEncode() {
		return encode;
	}

	/**
	 * 行番号情報を、出力する(true)/しない(false)を指定します。
	 *
	 * 通常のﾌｫｰﾏｯﾄでは、各行の先頭に行番号を出力します。
	 * これは、#NAME 属性を使用する場合には、必ず出力する必要があります。
	 * (#NAME 属性は、読み取り時には、必須です。)
	 * この、先頭の行番号が不要な場合(つまり、他のｼｽﾃﾑへのﾃﾞｰﾀ出力、
	 * このｼｽﾃﾑでは、#NAME 属性が出力されないため、読み込みできません。)
	 * この行番号を出力しないようにできます。
	 * 初期値は、true(出力する) です。
	 *
	 * @og.rev 3.7.0.2 (2005/02/14) 新規追加
	 *
	 * @param	useNumber	行番号情報を [true:出力する/false:しない]
	 */
	@Override	// TableWriter
	public void setUseNumber( final boolean useNumber ) {
		this.useNumber = useNumber;
	}

	/**
	 * 行番号情報を、出力する(true)/しない(false)を返します。
	 *
	 * 通常のﾌｫｰﾏｯﾄでは、各行の先頭に行番号を出力します。
	 * これは、#NAME 属性を使用する場合には、必ず出力する必要があります。
	 * (#NAME 属性は、読み取り時には、必須です。)
	 * この、先頭の行番号が不要な場合(つまり、他のｼｽﾃﾑへのﾃﾞｰﾀ出力、
	 * このｼｽﾃﾑでは、#NAME 属性が出力されないため、読み込みできません。)
	 * この行番号を出力しないようにできます。
	 * 初期値は、true(出力する) です。
	 *
	 * @og.rev 3.7.0.2 (2005/02/14) 新規追加
	 *
	 * @return	行番号情報を、出力する(true)/しない(false)
	 */
	protected boolean isUseNumber() {
		return useNumber;
	}

	/**
	 * ﾊﾟﾗﾒｰﾀｰﾘｽﾄをｾｯﾄします。
	 * 内部は、HybsEntry ｸﾗｽを持っています。
	 * 引数が、null の場合は、何もしません。
	 * ※ このｸﾗｽでは実装されていません。
	 *
	 * @og.rev 4.0.0.0 (2005/01/31) 新規追加
	 *
	 * @param	listParam	ﾊﾟﾗﾒｰﾀｰﾘｽﾄ
	 */
	@Override	// TableWriter
	public void setParam( final List<HybsEntry> listParam ) {
		final String errMsg = "param は、XML追加機能ですので、使用できません。";
		throw new UnsupportedOperationException( errMsg );
	}

	/**
	 * 出力先ﾌｧｲﾙのｶﾗﾑ列を、外部(ﾀｸﾞ)よりCSV形式の文字列で指定します。
	 * ただし、指定のｶﾗﾑ名は、DBTableModel上に存在している必要があります。
	 *
	 * @og.rev 4.0.0.0 (2005/11/30) 新規追加
	 *
	 * @param	clms	出力先ﾌｧｲﾙのｶﾗﾑ列(CSV形式)
	 */
	@Override	// TableWriter
	public void setColumns( final String clms ) {
		columns = clms ;
	}

	/**
	 * 書き込み対象外のｶﾗﾑ列を、外部(ﾀｸﾞ)よりCSV形式で指定します。
	 *
	 * 指定するｶﾗﾑ名に対して、書き込み処理を行いません。
	 * ここで指定するｶﾗﾑ名は、検索したDBTableModel上に含まれる必要はありません。
	 * その場合は、ここでの指定は無視されます。
	 *
	 * @og.rev 6.1.0.0 (2014/12/26) omitNames 属性を追加
	 *
	 * @param	clms	書き込み対象外のｶﾗﾑ列(CSV形式)
	 */
	@Override	// TableWriter
	public void setOmitNames( final String clms ) {
		omitNames = clms;
	}

	/**
	 * ﾃﾞｰﾀの書き込み開始行番号を設定します(初期値:0)。
	 *
	 * TAB区切りﾃｷｽﾄやEXCEL等のﾃﾞｰﾀの書き込みの開始行番号を指定します。
	 * 属性名は、行を飛ばす処理ということで、readTable ﾀｸﾞと同じ名称です。
	 * ﾌｧｲﾙの先頭行が、0行としてｶｳﾝﾄしますので、設定値は、読み飛ばす
	 * 件数になります。(1と指定すると、1件読み飛ばし、2行目から読み込みます。)
	 * 行の読み飛ばしと、ｶﾗﾑの読み飛ばし(columns)、refFileURL、refFilename、
	 * refSheetName とともに使用すれば、ある程度のﾚｲｱｳﾄ設定が可能です。
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 5.7.9.0 (2014/08/08) 新規作成
	 * @og.rev 6.0.2.0 (2014/09/19) isExcel() で判定しているので、Exception は投げない。
	 *
	 * @param	skipRowCount	書き込み開始行番号
	 */
	@Override	// TableWriter
	public void setSkipRowCount( final int skipRowCount ) {
		if( skipRowCount != 0 ) {
			final String errMsg = "skipRowCount は、EXCEL専用機能なので使用できません。[" + skipRowCount + "]";
			System.err.println( errMsg );
		}
	}

	/**
	 * EXCEL出力時に、ﾃﾞｰﾀを書き込んだ範囲に罫線を入れるかどうかを指定します。
	 *
	 * ﾃﾞｰﾀを書き込んでEXCELを作成しても、ﾉｰﾏﾙのｾﾙに値がｾｯﾄされている
	 * だけなので、ある程度加工が必要です。
	 * そこで、ﾃﾞｰﾀのｾｯﾄされたｾﾙに罫線を入れることで、それなりのﾃﾞｰﾀが
	 * 出力された感じになります。
	 * この設定と、useAutoCellSize="true" で、ｾﾙの幅を自動調整すれば、見栄えが良くなります。
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 6.0.2.0 (2014/09/19) 新規作成
	 *
	 * @param	useCellStyle	罫線を入れるかどうか(true:入れる/false:入れない)
	 * @see		#setUseAutoCellSize( boolean )
	 */
	@Override	// TableWriter
	public void setUseCellStyle( final boolean useCellStyle ) {
		final String errMsg = "useCellStyle は、EXCEL専用機能なので使用できません。[" + useCellStyle + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時に、ｾﾙの幅をﾃﾞｰﾀの幅に自動的に合わせるかどうかを指定します。
	 *
	 * ﾃﾞｰﾀを書き込んでEXCELを作成しても、ﾉｰﾏﾙのｾﾙに値がｾｯﾄされている
	 * だけなので、ある程度加工が必要です。
	 * そこで、ﾃﾞｰﾀのｾｯﾄされたｾﾙの幅を自動調整することで、それなりのﾃﾞｰﾀが
	 * 出力された感じになります。
	 * この設定と、useCellStyle="true" で、ｾﾙの罫線を自動設定すれば、見栄えが良くなります。
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 6.0.2.0 (2014/09/19) 新規作成
	 *
	 * @param	useAutoCellSize	ﾃﾞｰﾀの幅に自動的に合わせるかどうか(true:自動調整/false:何もしない)
	 * @see		#setUseCellStyle( boolean )
	 */
	@Override	// TableWriter
	public void setUseAutoCellSize( final boolean useAutoCellSize ) {
		final String errMsg = "useAutoCellSize は、EXCEL専用機能なので使用できません。[" + useAutoCellSize + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時に、ｾﾙの有効範囲を設定するかどうかを指定します。
	 *
	 * ｾﾙの有効範囲というのは、EXCELでの 空行、空列の存在しない範囲を指します。
	 * 通常、空行でも、ﾃﾞｰﾀとして残っている場合は、EXCELのｾﾙｵﾌﾞｼﾞｪｸﾄは存在します。
	 * ここで、useActiveWorkbook="true" とすると、空行、空列を削除します。
	 *
	 * 雛形を使用した場合は、ﾃﾞｰﾀより多めに設定した計算などは、この処理で
	 * 削除されますので、ﾃﾞｰﾀｻｲｽﾞにﾌｨｯﾄさせることができます。
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 6.0.2.0 (2014/09/19) 新規作成
	 *
	 * @param	useActiveWorkbook	ｾﾙの有効範囲を設定するかどうか(true:設定する/false:そのまま)
	 */
	@Override	// TableWriter
	public void setUseActiveWorkbook( final boolean useActiveWorkbook ) {
		final String errMsg = "useActiveWorkbook は、EXCEL専用機能なので使用できません。[" + useActiveWorkbook + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時に、ｼｰﾄ変更するｷｰとなるｶﾗﾑ名を指定します(このｶﾗﾑの値がｼｰﾄ名になります)。
	 *
	 * EXCEL帳票では、帳票雛形に、PAGE_BRAKE ｷｰを設定しましたが、TableWriterでは、
	 * ﾒﾓﾘ上のｶﾗﾑの値が変更したときに、ｼｰﾄ変更させることができます。
	 * このｶﾗﾑの値がｷｰﾌﾞﾚｲｸすると、新しいｼｰﾄに書き出し始めます。
	 * ｼｰﾄ名は、このｶﾗﾑの値（ｷｰﾌﾞﾚｲｸする値）です。
	 *
	 * 雛形ﾌｧｲﾙを使用する場合、雛形ｼｰﾄもｷｰﾌﾞﾚｲｸに伴って、＋1されます。
	 * つまり、雛形ｼｰﾄとﾃﾞｰﾀｼｰﾄは同時に変更されます。
	 * ただし、雛形ｼｰﾄは、最後の雛形ｼｰﾄで止まります。
	 * これは、雛形ｼｰﾄにﾍｯﾀﾞｰ雛形とﾎﾞﾃﾞｨ雛形を用意しておき、最初のｷｰﾌﾞﾚｲｸで
	 * ﾍｯﾀﾞｰからﾎﾞﾃﾞｨの書き込みにﾁｪﾝｼﾞするｲﾒｰｼﾞで使用できます。
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 6.0.2.0 (2014/09/19) 新規作成
	 *
	 * @param	pageBreakColumn	ｼｰﾄ変更するｷｰとなるｶﾗﾑ名を指定
	 * @see		#setFileBreakColumn( String )
	 */
	@Override	// TableWriter
	public void setPageBreakColumn( final String pageBreakColumn ) {
		final String errMsg = "pageBreakColumn は、EXCEL専用機能なので使用できません。[" + pageBreakColumn + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時に、ﾌｧｲﾙ名を変更するｷｰとなるｶﾗﾑ名を指定します(このｶﾗﾑの値がﾌｧｲﾙ名になります)。
	 *
	 * EXCEL帳票では、ﾒﾓﾘ上のｶﾗﾑの値が変更したときに、ﾌｧｲﾙ名を変更することができます。
	 * このｶﾗﾑの値がｷｰﾌﾞﾚｲｸすると、新しいﾌｧｲﾙに書き出し始めます。
	 * ﾌｧｲﾙ名は、このｶﾗﾑの値（ｷｰﾌﾞﾚｲｸする値）＋ 元の出力ﾌｧｲﾙ名の拡張子(.xlsなど)です。
	 * この設定を使用する場合は、出力ﾌｧｲﾙ名は無視されますが、拡張子だけは使用されます。
	 *
	 * 雛形ﾌｧｲﾙを使用する場合、雛形ﾌｧｲﾙもｷｰﾌﾞﾚｲｸに伴って、再利用されます。
	 * 例えば、pageBreakColumn と併用する場合、ｷｰﾌﾞﾚｲｸで雛形ｼｰﾄも最初から適用になります。
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 6.0.2.0 (2014/09/19) 新規作成
	 *
	 * @param	fileBreakColumn	ﾌｧｲﾙ名を変更するｷｰとなるｶﾗﾑ名を指定
	 * @see		#setPageBreakColumn( String )
	 */
	@Override	// TableWriter
	public void setFileBreakColumn( final String fileBreakColumn ) {
		final String errMsg = "fileBreakColumn は、EXCEL専用機能なので使用できません。[" + fileBreakColumn + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時に、Hyperlinkを作成するｷｰとなるｶﾗﾑ名と値となるｶﾗﾑ名を指定します。
	 *
	 * ここで、作成するﾊｲﾊﾟｰﾘﾝｸは、EXCELのｼｰﾄに対するﾊｲﾊﾟｰﾘﾝｸです。
	 * それ以外のﾘﾝｸ(本当のURLやﾌｧｲﾙ等)のﾘﾝｸは(今は)作成できません。
	 * ﾊｲﾊﾟｰﾘﾝｸを作成するには、①作成するｶﾗﾑ と ②作成する値 が必要です。
	 * このﾒｿｯﾄﾞで設定するのは、｢①:②｣という形式でｶﾗﾑ名を指定します。
	 * ②がなければ、①と同じとします。
	 * ②の値のｼｰﾄの存在有無は、無視します。ﾊｲﾊﾟｰﾘﾝｸを作成するｼｰﾄを作成する前に
	 * ﾊｲﾊﾟｰﾘﾝｸを作成するｹｰｽが存在します。
	 * (例えば、各ｼｰﾄへのﾘﾝｸを持った一覧を作成してから、明細の各ｼｰﾄを作成する様なｹｰｽ)
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 6.0.2.0 (2014/09/19) 新規作成
	 *
	 * @param	hyperLinkColumn	Hyperlinkを作成するｷｰとなるｶﾗﾑ名と値となるｶﾗﾑ名を指定
	 */
	@Override	// TableWriter
	public void setHyperLinkColumn( final String hyperLinkColumn ) {
		final String errMsg = "hyperLinkColumn は、EXCEL専用機能なので使用できません。[" + hyperLinkColumn + "]";
		System.err.println( errMsg );
	}

	/**
	 * EXCEL出力時に、Sheet一覧を先頭Sheetに作成する場合のSheet名を指定します。
	 *
	 * これは、Workbook に含まれる Sheet 一覧を作成する場合に、利用可能です。
	 * なお、この機能は、TableWriter_Excel のみに実装します。
	 *
	 * @og.rev 6.0.2.0 (2014/09/19) 新規作成
	 *
	 * @param	sheetName	EXCELﾌｧｲﾙのｼｰﾄ名
	 */
	@Override	// TableWriter
	public void setAddTitleSheet( final String sheetName ) {
		final String errMsg = "addTitleSheet は、EXCEL専用機能なので使用できません。[" + sheetName + "]";
		System.err.println( errMsg );
	}

	/**
	 * 書込処理でｺｰﾄﾞﾘｿｰｽのﾗﾍﾞﾙ変換を行うかどうか[true/false]を指定します。
	 *
	 * ｺｰﾄﾞﾘｿｰｽをそのままの値で出力すると、数字や記号になり何が書かれているのか
	 * 不明になります。
	 * これは、ｺｰﾄﾞﾘｿｰｽをﾗﾍﾞﾙに変換して出力するかどうかを指定します。
	 * 当然、ｺｰﾄﾞはﾕﾆｰｸですが、ﾗﾍﾞﾙはﾕﾆｰｸになるかどうか保障はされていませんので
	 * TableReader 系で読み込む場合には、ﾘｽｸが発生します。
	 * また、TableReader 系で読み込む場合にも、ﾗﾍﾞﾙからｺｰﾄﾞを求める逆変換を行うように、
	 * setUseRenderer ﾒｿｯﾄﾞで指定する必要があります。
	 *
	 * 従来は、TableWriter 系に、TableWriter_Renderer 系のｸﾗｽを作って対応していましたが、
	 * このﾒｿｯﾄﾞの属性値のﾌﾗｸﾞで、制御します。
	 *
	 * @og.rev 5.2.1.0 (2010/10/01) 新規作成
	 *
	 * @param	useRenderer	ｺｰﾄﾞﾘｿｰｽのﾗﾍﾞﾙ変換を行うかどうか [true:行う/false:行わない]
	 */
	@Override	// TableWriter
	public void setUseRenderer( final boolean useRenderer ) {
		this.useRenderer = useRenderer;
	}

	/**
	 * 書き出し処理でｺｰﾄﾞﾘｿｰｽのﾗﾍﾞﾙ変換を行うかどうかを返します。
	 * 初期値は、false(行わない) です。
	 *
	 * @og.rev 5.2.1.0 (2010/10/01) 新規作成
	 *
	 * @return	ﾗﾍﾞﾙ変換を行うかどうか(true:行う/false:行わない)
	 */
	protected boolean isUseRenderer() {
		return useRenderer ;
	}

	/**
	 * ﾃﾞｰﾀを ﾀﾞﾌﾞﾙｺｰﾃｰｼｮﾝ(")で囲みます。
	 *
	 * この処理では、前後をﾀﾞﾌﾞﾙｸｵｰﾄで囲うため、改行の有無は判定する
	 * 必要はありません。
	 * ﾀﾞﾌﾞﾙｸｵｰﾄを含む場合は、その直前にﾀﾞﾌﾞﾙｸｵｰﾄを強制的に追加します。
	 *
	 * @param	data	元のString文字列
	 * @return	ﾀﾞﾌﾞﾙｺｰﾃｰｼｮﾝで囲まれた文字列
	 * @og.rtnNotNull
	 */
	protected String quotation( final String data ) {
		return "\"" + StringUtil.replace( data,"\"","\"\"" ) + "\"" ;
	}

	/**
	 * ﾃﾞｰﾀに対して 改行、ﾀﾞﾌﾞﾙｸｵｰﾄ等の処理を行います。
	 *
	 * これは、ﾃﾞｰﾀの前後をﾀﾞﾌﾞﾙｸｵｰﾄで括る quotation(String)と異なり
	 * 前後には、ﾀﾞﾌﾞﾙｸｵｰﾄ を追加しません。
	 * ただし、以下の２つの条件で処理を行います。
	 * 1) 改行を含む場合は、ﾀﾞﾌﾞﾙｸｵｰﾄを強制的に前後に追加する。
	 * 2) ﾀﾞﾌﾞﾙｸｵｰﾄを含む場合は、その直前にﾀﾞﾌﾞﾙｸｵｰﾄを強制的に追加する。
	 *
	 * @og.rev 5.2.2.0 (2010/11/01) 新規追加
	 *
	 * @param	data	元のString文字列
	 *
	 * @return	改行、ﾀﾞﾌﾞﾙｸｵｰﾄ等の処理
	 */
	protected String quotation2( final String data ) {
		String rtn = StringUtil.replace( data,"\"","\"\"" );
		if( rtn != null && rtn.indexOf( CR ) >= 0 ) {
			rtn = "\"" + rtn + "\"" ;
		}

		return rtn ;
	}

	/**
	 * keys の整合性ﾁｪｯｸを行います。
	 *
	 * ｷｰかkeysMapのどちらかが null の場合は、何もしません。
	 * ﾁｪｯｸするｷｰは、内部で大文字に変換しておきます。
	 * keysMap に登録するｷｰは、大文字で登録しておいてください。
	 * ここのﾁｪｯｸでは、ｷｰﾏｯﾌﾟに存在しなければ、ｴﾗｰになります。
	 *
	 * @og.rev 5.6.6.1 (2013/07/12) keys の整合性ﾁｪｯｸを行います。
	 * @og.rev 6.4.3.4 (2016/03/11) forﾙｰﾌﾟを、forEach ﾒｿｯﾄﾞに置き換えます。
	 * @og.rev 7.0.1.0 (2018/10/15) XHTML → HTML5 対応(空要素の、"／＞" 止めを、"＞" に変更します)。
	 *
	 * @param	key		ﾁｪｯｸするｷｰ文字列(null の場合は、処理しない)
	 * @param	keysMap	利用可能なｷｰのﾏｯﾌﾟ(大文字に統一)
	 */
	protected void checkParam( final String key,final Map<String,String> keysMap ) {
		// key か keysMap かどちらかが null の場合は、処理を行わない。
		if( key == null || keysMap == null ) { return; }

		final String upKey = key.toUpperCase(Locale.JAPAN);

		if( ! keysMap.containsKey( upKey ) ) {
//			final String BR = "<br />" + CR ;
			final String BR = "<br>" + CR ;				// 7.0.1.0 (2018/10/15)
			final StringBuilder errMsg = new StringBuilder( BUFFER_MIDDLE )
			// 6.0.2.5 (2014/10/31) char を append する。
				.append( BR )
				.append( "指定のキーは、この tableWriter では、使用できません。"  ).append( BR )
				.append( "  class=[" ).append( getClass().getName() ).append( ']' ).append( BR )
				.append( "  key  =[" ).append( key				    ).append( ']' ).append( BR )
				.append( "  ======== usage keys ======== "						  ).append( BR ) ;

			// 6.4.3.4 (2016/03/11) forﾙｰﾌﾟを、forEach ﾒｿｯﾄﾞに置き換えます。
			keysMap.forEach( (k,v) -> errMsg.append( ' ' ).append( k ).append( ':' ).append( v ).append( BR ) );
			errMsg.append( "  ============================ " ).append( BR );

			throw new HybsSystemException( errMsg.toString() );
		}
	}

	/**
	 * ﾃﾞﾊﾞｯｸﾞ情報を出力するかどうか[true:する/false:しない]を指定します。
	 *
	 * EXCELなどを書き出す場合、ｼｰﾄﾌﾞﾚｲｸやﾌｧｲﾙﾌﾞﾚｲｸ時の行番号が、検索時の行番号と
	 * 異なる為、ｴﾗｰ時の判定が難しくなります。
	 * そこで、どうしてもわからなくなった場合に備えて、ﾃﾞﾊﾞｯｸﾞ情報を出力できるようにします。
	 * 通常は使用しませんので、設定を無視します。
	 * 初期値は、false:ﾃﾞﾊﾞｯｸﾞ情報を出力しない です。
	 *
	 * @og.rev 6.1.0.0 (2014/12/26) ﾃﾞﾊﾞｯｸﾞ情報を出力するかどうかを指定
	 *
	 * @param	useDebug	ﾃﾞﾊﾞｯｸﾞ出力するか [true:する/false:しない]
	 */
	@Override	// TableWriter
	public void setDebug( final boolean useDebug ) {
		this.useDebug = useDebug;
	}

	/**
	 * ﾃﾞﾊﾞｯｸﾞ情報を出力するかどうか[true:する/false:しない]を取得します。
	 *
	 * EXCELなどを書き出す場合、ｼｰﾄﾌﾞﾚｲｸやﾌｧｲﾙﾌﾞﾚｲｸ時の行番号が、検索時の行番号と
	 * 異なる為、ｴﾗｰ時の判定が難しくなります。
	 * そこで、どうしてもわからなくなった場合に備えて、ﾃﾞﾊﾞｯｸﾞ情報を出力できるようにします。
	 *
	 * @og.rev 6.1.0.0 (2014/12/26) ﾃﾞﾊﾞｯｸﾞ情報を出力するかどうかを指定
	 *
	 * @return	ﾃﾞﾊﾞｯｸﾞ出力 [true:する/false:しない]
	 */
	protected boolean isDebug() {
		return useDebug ;
	}
}
